/***************************************************************************
                          Game.cpp  -  description
                             -------------------
    begin                : Sat May 10 2003
    copyright            : (C) 2003 by Tyler Montbriand
    email                : tsm@accesscomm.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#define __GAME_CPP__

#ifdef WIN32
#include <windows.h>
#endif

#include <stdlib.h>
#include "Game.h"
#include "Crossfire.h"

/*! \brief The game object. */
static Crossfire *game=NULL;

/**
 *  Creates the game object, or resets it if it already exists. 
 */
void Game_Init(void)
{
  if(game==NULL)
  {
    game = new Crossfire();
  }
  else
  {
    game->Reset();
  }
}

/**
 *  Returns the title screen image from the game object, if it exists,
 *  or NULL if it does not.
 */
VCS_Surface *Game_GetTitle()
{
  if(game==NULL) return(NULL);
  else return(game->GetTitle());
}

/**
 *  Attempts to run the game, creating it if it doesn't exist or
 *  resetting it if it does.  Frametick is the requested delay
 *  (in ticks) between frames, i.e. 1000/FPS.
 *
 *  Returns -1 if the game object didn't initialize properly,
 *  0 otherwise.
 */
int Game(int frametick)
{
  Game_Init();
  
  if(!(game->Good())) return(-1);
    
  game->Run(frametick);
  
  return(0);
}

/**
 *  Destroys the game object, if it exists.
 */
void Game_Quit()
{
  if(game!=NULL)
  {
    delete game;
    game=NULL;
  }
}
