/***************************************************************************
                          GameGrid.h  -  description
                             -------------------
    begin                : Sun May 11 2003
    copyright            : (C) 2003 by Tyler Montbriand
    email                : tsm@accesscomm.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef __GAMEGRID_H__
#define __GAMEGRID_H__

#include <spritelib/spritelib.h>

/*! \brief Pixel width of each grid square, in fixed point */
#define GRID_X    TO_SFXP(17)
/*! \brief Pixel height of each grid square, in fixed point */
#define GRID_Y    GRID_X
/*! \brief X pixel offset of the game grid, in fixed point */
#define GOFF_X    TO_SFXP(4)
/*! \brief Y pixel ofset of the game grid, in fixed point */
#define GOFF_Y    TO_SFXP(7)
/*! \brief Number of horizontal squares in the game board */
#define GCOUNT_X  15
/*! \brief Number of vertical squares in the game board */
#define GCOUNT_Y  13
/*! \brief fixed-point X coordinate of the rightmost game squares*/
#define GMAX_X  ((GRID_X*(GCOUNT_X-1))+GOFF_X)
/*! \brief fixed-point X coordinate of the leftmost game squares */
#define GMIN_X  (GOFF_X)
/*! \brief fixed-point Y coordinate of the bottom-most game squares */
#define GMAX_Y  ((GRID_Y*(GCOUNT_Y-1))+GOFF_Y)
/*! \brief fixed-point Y coordinate of the topmost game squares */
#define GMIN_Y  (GOFF_Y)

/*! \brief Minimum sound panning value */
#define G_MIN_PAN 60
/*! \brief Maximum sound panning value */
#define G_MAX_PAN 60

extern "C" {
/*! \brief Converts a game-square X count to a signed fixed point value.  */
Sfxp    G_XPos(int xcount);
/*! \brief Converts a game-square Y count to a signed fixed point value.  */
Sfxp    G_YPos(int ycount);
/*! \brief Returns the fixed-point distance between x and a game column   */
Sfxp    G_DistX(Sfxp x, int xcount);
/*! \brief Returns the fixed-point distance between x and a game row      */
Sfxp    G_DistY(Sfxp y, int ycount);
/*! \brief Calculates new X coordinate when travelling on game grid */
Sfxp    G_SnapX(Sfxp pos, Sfxp speed, int dtick, int xcount);
/*! \brief Calculates new Y coordinate when travelling on game grid */
Sfxp    G_SnapY(Sfxp pos, Sfxp speed, int dtick, int ycount);
/*! \brief Generates a random fixed-point x/y position on the grid. */
void    G_GetRandPos(Sfxp *x, Sfxp *y);
/*! \brief Calculates the panning angle for a fixed-point x coordinate.*/
Sint16  G_CalcPan(Sfxp x);
/*! \brief Returns the absolute value of an integer or signed fixed-point */
int     Abs(int val);
/*! \brief Returns a random value between min and max. */
Sint32  RandRange(Sint32 min, Sint32 max);
/*! \brief Selects a random value from retval based on the values in odds. */
int SwitchOdds(int odds[], int retval[]);
}

#endif/*__GAMEGRID_H__*/
