/***************************************************************************
                          GridSprite.cpp  -  description
                             -------------------
    begin                : Tue Jun 3 2003
    copyright            : (C) 2003 by Tyler Montbriand
    email                : tsm@accesscomm.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "GridSprite.h"


/**
 *  Default constructor.  Sets position and speed to zero
 *  and marks itself as deleteable.
 */
GridSprite::GridSprite()
{
  SnapTo(0,0);
  speed=0;
  SetFlags(SPRITE_DELETEABLE);
}

/**
 *  Sets starting position and speed, and marks itself as deleteable.
 */
GridSprite::GridSprite(int igridx, int igridy, Sfxp ispeed)
{
  SnapTo(igridx,igridy);
  speed=ispeed;
  SetFlags(SPRITE_DELETEABLE);
}

/**
 *  Moves the sprite's current position and destination to
 *  the specified grid coordinates.
 */
void GridSprite::SnapTo(int x, int y)
{
  gridx=x;
  gridy=y;
  pos.SetFxp(G_XPos(gridx),G_YPos(gridy));
}

Sprite *GridSprite::Clone()
{
  GridSprite *spr=new GridSprite(0,0,0);
  (*spr)=(*this);
  spr->ClearFlags(SPRITE_OWNSURFACE|SPRITE_OWNBITMASK);
  return(spr);
}

/**
 *  Calculates the bitfield representing the direction(s)
 *  the sprite is travelling in.
 */
int GridSprite::Direction()
{
  int dx=G_DistX(pos.Xfxp(), gridx);
  int dy=G_DistY(pos.Yfxp(), gridy);
  int dir=0;

  if(dx)
  {
    if(dx<0) dir|=DIR_LEFT;
    else     dir|=DIR_RIGHT;
  }

  if(dy)
  {
    if(dy<0) dir|=DIR_UP;
    else     dir|=DIR_DOWN;
  }

  return(dir);
}

/**
 *  Calculates the sprite's new position, ensures the velocity is zero,
 *  then calls the base class update function.
 */
void GridSprite::Update(Sint32 dtick)
{
  vel.SetFxp(0,0);
  prevpos=pos;
  pos.SetFxp(Dx(dtick),Dy(dtick));
  Sprite::Update(dtick);
}

/*********Functions - not class members, global functions */

/**
 *  Rotates the direction(s) passed ninety degrees clockwise.
 */
int RotateDir(int dir)
{
  int dirout=0;
  
  if(dir&DIR_UP)        dirout|=DIR_RIGHT;
  else if(dir&DIR_DOWN) dirout|=DIR_LEFT;
  
  if(dir&DIR_RIGHT)     dirout|=DIR_DOWN;
  else if(dir&DIR_LEFT) dirout|=DIR_UP;

  return(dirout);
}

void Offset(int dir, int mag, int &x, int &y)
{
  if(dir&DIR_RIGHT)     x+=mag;
  else if(dir&DIR_LEFT) x-=mag;
  
  if(dir&DIR_UP)        y-=mag;
  else if(dir&DIR_DOWN) y+=mag;
}
