/***************************************************************************
                          GridSprite.h  -  description
                             -------------------
    begin                : Sun May 11 2003
    copyright            : (C) 2003 by Tyler Montbriand
    email                : tsm@accesscomm.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef __GRIDSPRITE_H__
#define __GRIDSPRITE_H__

#ifndef __cplusplus
#error "Needs C++ compilation"
#endif

#include <spritelib/spritelib.h>
#include "GameGrid.h"

/*! \brief Bit that represents travelling left.   */
#define DIR_LEFT  0x0001
/*! \brief Bit that represents travelling right.  */
#define DIR_RIGHT 0x0002
/*! \brief Bit that represents travelling up.     */
#define DIR_UP    0x0004
/*! \brief Bit that represents travelling down.   */
#define DIR_DOWN  0x0008

/*! \brief Rotates the passed direction 90 degrees clockwise.       */
extern "C" int RotateDir(int dir);
/*! \brief Offsets the X/Y coordinates mag amount in dir direction. */
extern "C" void Offset(int dir, int mag, int &x, int &y);

/*! \brief GridSprite - a sprite that travels relative to the game grid */
/**
 *  Unlike ordinary sprites, which move via position and velocity,
 *  GridSprites move via position, destination, and speed.
 *  This destination is given in terms of grid coordinates, not
 *  fixed point values.
 */
class GridSprite:public Sprite
{
public:
  /*! \brief Default constructor */
  GridSprite();
  /*! \brief Constructor specifying starting position and speed */
  GridSprite(int igridx, int igridy, Sfxp ispeed);
  /*! \brief Moves the sprite directly to the specified grid coordinates  */
  void SnapTo(int x, int y);
  /*! \brief Creates a perfect clone that shares but doesn't own data     */
  virtual Sprite *Clone();
  /*! \brief Returns a bitfield denoting the direction it's travelling in */
  int Direction();
  /*! \brief Calculates movement */
  virtual void Update(Sint32 dtick);

  /*! \brief Returns current destination's X grid coordinate */
  inline int GridX() { return(gridx);  }
  /*! \brief Returns current destination's Y grid coordinate */  
  inline int GridY() { return(gridy);  }

protected:
  /*! \brief Returns nonzero if the sprite is moving, zero otherwise. */
  inline int Moving() { return(Dx() | Dy()); }
  /*! \brief Returns the sprite's new fixed-point X position */
  inline Sfxp Dx(int dtick=1)
  { return(G_SnapX(pos.Xfxp(),speed,dtick,gridx));  }
  /*! \brief Returns the sprite's new fixed-point Y position */
  inline Sfxp Dy(int dtick=1)
  { return(G_SnapY(pos.Yfxp(),speed,dtick,gridy));  }

  /*! \brief Destination grid coordinates */
  int gridx,gridy;
  /*! \brief Speed */
  Sfxp speed;  
};

#endif/*__GRIDSPRITE_H__*/
