/***************************************************************************
                          InitSDL.cpp  -  description
                             -------------------
    begin                : Sat May 10 2003
    copyright            : (C) 2003 by Tyler Montbriand
    email                : tsm@accesscomm.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "InitSDL.h"

extern "C" int InitVideo(int x, int y, int fullscreen);
VCS_Surface *__screen=NULL;
int          __fullscreen=0;

/**
 *  Deinitializes SDL_mixer, then SDL.
 */
void Quit()
{
  Mix_CloseAudio();
  VCS_Quit();
}

/**
 *  If video is windowed, attempts to change to fullscreen.
 *  If video is fullscreen, attempts to change to windowed.
 *  Upon failure, it attempts to set the screen to it's previous
 *  state before returning -1.  0 is returned on success.
 */
int ToggleFullscreen()
{
  int x=-1,y=-1;
  int fs=__fullscreen;
  
  if(__screen==NULL)  return(-1);

  x=__screen->w;
  y=__screen->h;

  if(InitVideo(x,y,!fs)<0)
  {
    return(InitVideo(x,y,fs));
  }
  else
  {
    return(0);
  }
}

/**
 *  Attempts to set the video mode.
 */
int InitVideo(int x, int y, int fullscreen)
{
  if(fullscreen)
  {
    __fullscreen=1;
    __screen=VCS_SetVideoMode(x,y,DEFAULT_BPP,SDL_ANYFORMAT|SDL_FULLSCREEN);
  }
  else
  {
    __fullscreen=0;
    __screen=VCS_SetVideoMode(x,y,DEFAULT_BPP,SDL_ANYFORMAT);
  }

  if(__screen==NULL) return(-1);
  else               return(0);
}

/**
 *  Attempts to initialize SDL, set the video mode, then
 *  initialize SDL_mixer.  Returns 0 on success, -1 on failure.
 */
int Init(int x, int y, int fullscreen)
{
  if(SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO|SDL_INIT_TIMER)<0)
  {
    fprintf(stderr,"Init():  Failed to initialize SDL\n");
    return(-1);
  }

  if(Mix_OpenAudio(AUDIO_RATE,AUDIO_TYPE,AUDIO_CHAN,AUDIO_BUF)<0)
  {
    fprintf(stderr,"Init():  Failed to initialize SDL_mixer\n");
    SDL_Quit();
    return(-1);
  }
  
  if(InitVideo(x,y,fullscreen)<0)
  {
    fprintf(stderr,"Init():  Failed to set video mode(%d,%d,%d)\n",
                                                      x,y,fullscreen);
    Mix_CloseAudio();    
    VCS_Quit();    
    return(-1);
  }

  return(0);
}


void PushUserEvent(int code, void *data1, void *data2)
{
  SDL_Event event;
  event.type=SDL_USEREVENT;
  event.user.code=code;
  event.user.data1=data1;
  event.user.data2=data2;
  SDL_PushEvent(&event);
}
