/***************************************************************************
                          LoadData.h  -  description
                             -------------------
    begin                : Tue May 13 2003
    copyright            : (C) 2003 by Tyler Montbriand
    email                : tsm@accesscomm.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef __LOADDATA_H__
#define __LOADDATA_H__

#ifndef __cplusplus
#error "Requires C++ compilation"
#endif

template<class T>
inline void ClearList(const char *names[], T *items[])
{
  int n=-1;
  if((names==NULL)||(items==NULL)) return;

  for(n=0; names[n]!=NULL; n++) items[n]=NULL;  
}

template<class T>
inline T **FreeList(const char *names[], T **items)
{
  int n=-1;
  if((names==NULL)||(items==NULL)) return(NULL);

  for(n=0; names[n]!=NULL; n++)
  {
    if(items[n]!=NULL)
    {
      delete items[n];
      items[n]=NULL;
    }
  }

  delete items;
  return(NULL);
}

template<class T>
inline int LoadList(const char *names[],const char *path, T *items[], bool fail_on_error=true)
{
  int retval=1;
  
  int n=-1;
  if((names==NULL)||(items==NULL)) return(0);

  ClearList<T>(names,items);

  for(n=0; names[n]!=NULL; n++)
  {
    items[n]=new T(names[n],path);
    
    if(!items[n]->Good())
    {
      fprintf(stderr,"Error - couldn't load %s\n",names[n]);
      if(fail_on_error)
      {
        FreeList<T>(names,items);
        return(0);
      }
      else
      {
        retval=0;
      }
    }
  }

  return(retval);
}

template<class T>
inline T **CreateList(const char *names[], const char *path="",
                      bool fail_on_error=false)
{
  int n=-1;
  T **data=NULL;
  if(names==NULL) return(NULL);

  for(n=0; names[n]!=NULL; n++) { }

  if(n==0) return(NULL);
  data = new T *[n];

  ClearList<T>(names,data);

  if(!LoadList<T>(names,path,data,fail_on_error))
  {
    data=FreeList<T>(names,data);
    return(NULL);
  }
  else
  {
    return(data);
  }
}

#endif/*__LOADDATA_H__*/
