/***************************************************************************
                          Music.h  -  Singleton class.  Only one Music
    object is created, and it is shared amongst all code that uses it.
    Threadsafe.
                             -------------------
    begin                : Sat May 10 2003
    copyright            : (C) 2003 by Tyler Montbriand
    email                : tsm@accesscomm.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef __MUSIC_H__
#define __MUSIC_H__

#ifndef __cplusplus
#error "C++ compilation needed for Music.h"
#endif

#include "InitSDL.h"
#include <SDL/SDL_thread.h>
#include <stdlib.h>

class Music
{
public:
  /*! \brief Static accessor, returns the one and only music object */
  /**
   *  Initializes the music object if it's not, and returns the singular

   *  music object.  This function works recursively, i.e. if it gets
   *  called 3 times, it will take 3 calls to ~Music() to deinitialize it.
   */
  static Music &GetObject()
  {
    if(!music_init)
    {
      lock=SDL_CreateMutex();
    }
  
    /* Increment number of instances in existence */
    music_init++;

    return(music_object);
  }

  /*! \brief Attempts to play the specified music file  */
  int Play(const char *fname, int loops=-1, int fadein=0, int pos=0);
  /*! \brief Stops the currently playing song */  
  int Stop();
  /*! \brief Returns 1 if music is playing, 0 if not. */  
  int Playing();
  /*! \brief Public destructor, deinitializes the music object  */
  ~Music();
private:
  /*! \brief Private constructor prevents creation of new music objects.  */
  Music();

  /*! \brief Returns 1 if music data is loaded, 0 if not. */
  int Loaded();
  /*! \brief Attempts to load the specified music file. */  
  int Load(const char *fname);
  /*! \brief Stops and unloads currently loaded or playing music  */  
  int Free();

  inline void Lock()   { SDL_mutexP(lock); }
  inline void Unlock() { SDL_mutexV(lock); }

  static Music      music_object;
  static int        music_init;
  static Mix_Music *music_data;
  static SDL_mutex *lock;
};

#endif/*__MUSIC_H__*/
