/****************************************************************************
                             -------------------
    begin                : Mon May 12 2003
    copyright            : (C) 2003 by Tyler Montbriand
    email                : tsm@accesscomm.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef __SAMPLE_H__
#define __SAMPLE_H__

#include "InitSDL.h"
#include <spritelib/spritelib.h>

/*! \brief Simple class to load sound files */
class Sample
{
public:
  /*! \brief Constructor, takes filename, and optionally a path */
  Sample(const char *fname, const char *path="")
  {
    char buf[300]={'\0'};
    
    data=Mix_LoadWAV(SprMakePath(fname,path,buf));
  }
  
  /*! \brief Operator to access the loaded sound data, if any */
  operator Mix_Chunk *()
  {
    return(data);
  }

  /*! \brief  Returns true if the sound got loaded, false if not. */
  bool Good()
  {
    return(data!=NULL);
  }

  /*! \brief Destructor, frees any data loaded  */
  ~Sample()
  {
    if(data!=NULL) Mix_FreeChunk(data);
  }
  
private:
  /*! \brief Pointer to actual sound data  */
  Mix_Chunk *data;
};


/*! \brief A simple mixer class to interface with SDL_mixer */
class Mixer
{
public:
  /*! \brief Constructor, optionally taking the number of channels          */
  Mixer(int max_channels=16);
  /*! \brief Returns nonzero if any channels are playing, zero otherwise    */
  int Playing();
  /*! \brief Returns nonzero if specified sample is playing, zero otherwise */
  int Playing(Mix_Chunk *chunk);
  /*! \brief Halts all channels                                             */
  int Stop();
  /*! \brief Halts specified channel                                        */
  int Stop(int n);
  /*! \brief Returns nonzero if given channel is playing, zero otherwise    */
  int Playing(int n);
  /*! \brief Attempts to play the specified sample. */
  int Play(Mix_Chunk *data, Uint8 dist=0, Uint16 pos=0, int loop=0, int chan=-1);
  /*! \brief Destructor - stops all channels and frees data */    
  ~Mixer();
  
private:
  /*! \brief Pointers to sound data, to keep track of what's playing        */
  Mix_Chunk **channels;
  /*! \brief The maximum number of simultaneously playing channels          */
  int         maxchans;
};

#endif/*__SAMPLE_H__*/
